﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=2.9
@EndOfDesignText@
'Class module
'File: Dialogs.bas
'Purpose:
'Provide various User Dialogs
'MessageDialog = Show a messagewindow with title and message content. The message content accepts html.
'ConfirmDialog = YESNO dialog returning Boolean
'
'FXML files:
'MessageDialog.fxml
'ConfirmDialog.fxml
'TextDialog.fxml
'Controls:
'Control ID are used across the FXML files.
'Use in projects:
'-Copy Dialogs.bas to the Project Folder
'-Copy messagedialog.fxml, confirmdialog.fxml, textdialog.fxml to the project files folder
'-Add dialogs module to the project using "Project > Add Existing Module" and add the fxml files to the Files tab.
'Various Information:
'B4J Webview: http://www.basic4ppc.com/b4j/help/jfx.html#webview
'
Private Sub Class_Globals
	Private MessageDialogWidth As Int = 400			'Default messagedialog window width
	Private MessageDialogHeight As Int = 300		'Default messagedialog window height
	Private ConfirmDialogWidth As Int = 300			'Default confirmdialog window width
	Private ConfirmDialogHeight As Int = 200		'Default confirmdialog window width

	Private fx As JFX
	Private frm As Form	

	Private wvwMessage As WebView					'Shows the html formatted message
	Private msg As String							'Original message loaded in the webview - this to handle external link call (see under locationchanged)

	Private ConfirmResult As Boolean = False		'Confirm dialog result
	Private btnYes As Button						'Yes and No buttons of confirm dialogs
	Private btnNo As Button

	'TextDialog
	Private TextDialogWidth As Int = 600			'Default window width
	Private TextDialogHeight As Int = 400			'Default window width
	Private btnTextDialogSave As Node
	Private btnTextDialogCopy As Button
	Private taTextDialogMessage As TextArea
	Private lblTextDialogInformation As Label
End Sub

'Init class. The form and give the form a title and make it a utility form (toolbox type of form).
'The specific layout for the dialog is loaded by the respective dialog method, like MessageDialog.
Public Sub Initialize
	frm.Initialize("frm", MessageDialogWidth, MessageDialogHeight)
	frm.Resizable = False
	frm.SetFormStyle("UTILITY")
End Sub

'
'MESSAGE DIALOG
'

'Show a simple message dialog without a title
'Parameter: Message Content
Public Sub MessageDialog(Message As String)
	frm.WindowWidth = MessageDialogWidth 
	frm.WindowHeight = MessageDialogHeight
	frm.RootPane.LoadLayout("MessageDialog")
	wvwMessage.PrefHeight = MessageDialogHeight
	wvwMessage.PrefWidth = MessageDialogWidth 
	wvwMessage.LoadHtml(Message)
	msg = Message
	frm.ShowAndWait
End Sub

'Show a simple message dialog with title and message
'Parameter: Message Title, Message Content
Public Sub MessageDialog2(Title As String, Message As String)
	frm.WindowWidth = MessageDialogWidth 
	frm.WindowHeight = MessageDialogHeight
	frm.Title = Title
	frm.RootPane.LoadLayout("MessageDialog")
	wvwMessage.PrefHeight = MessageDialogHeight
	wvwMessage.PrefWidth = MessageDialogWidth 
	wvwMessage.LoadHtml(Message)
	msg = Message
	frm.ShowAndWait
End Sub

'Show the message dialog extended
'Parameter: Window Width and Height, Message Title, Message Content
'Note: To use default window size, set Width and Height to -1 
Public Sub MessageDialog3(w As Double, h As Double, Title As String, Message As String)
	frm.Title = Title
	frm.WindowWidth = MessageDialogWidth 
	frm.WindowHeight = MessageDialogHeight
	If w <> -1 Then frm.WindowWidth = w
	If h <> -1 Then frm.WindowHeight = h
	frm.RootPane.LoadLayout("MessageDialog")
	wvwMessage.PrefHeight = frm.WindowHeight
	wvwMessage.PrefWidth = frm.WindowWidth  
	wvwMessage.LoadHtml(Message)
	msg = Message
	frm.ShowAndWait
End Sub

'
'CONFIRM DIALOG
'

'Show confirm dialog with a Title and Message
'Parameter: Confirmation Title and Message
'Returns True (Button YES) or False (Button NO)
Public Sub ConfirmDialog(Title As String, Message As String) As Boolean
	frm.WindowWidth = ConfirmDialogWidth 
	frm.WindowHeight = ConfirmDialogHeight + 40
	frm.Title = Title
	frm.RootPane.LoadLayout("ConfirmDialog")
	wvwMessage.LoadHtml(Message)
	msg = Message
	frm.ShowAndWait
	Return ConfirmResult
End Sub

'Confirm YES button. Sets confirmResult to True.
Private Sub btnYes_Action
	ConfirmResult = True	
	frm.Close
End Sub

'Confirm NO button. Sets confirmResult to False.
Private Sub btnNo_Action
	ConfirmResult = False
	frm.Close
End Sub

'Handle the href to the authors homepage. target is to show in default browser and not in this message webview.
'Solution is bit clumsy, but anyhow it works
Private Sub wvwMessage_LocationChanged (Location As String)
	'Check if urllocationn contains authors homepage
	If Location.ToLowerCase.Contains("rwblinn.de") Then
		'Reload the original message = current content of the webview
		wvwMessage.LoadHtml(msg)
		'And call the external browser and show the authors homepage
		fx.ShowExternalDocument(Location)
	End If
End Sub

'Show a dialog where the content is shown in a textarea.
'The content can be set to editable.
'Parameter: Message Title, Message Content, Message Footer, Editable as boolean
Public Sub TextDialog(Title As String, Message As String, Footer As String, Editable As Boolean)
	frm.WindowWidth = TextDialogWidth 
	frm.WindowHeight = TextDialogHeight
	frm.Title = Title
	frm.RootPane.LoadLayout("TextDialog")
	lblTextDialogInformation.Text = Footer
	taTextDialogMessage.Editable = Editable
	taTextDialogMessage.Text = Message
	frm.ShowAndWait
End Sub

'TextDialog SAVE button.
Private Sub btnTextDialogSave_Action
	If taTextDialogMessage.Text.Length = 0 Then Return		'If the textarea has no content, return
	Dim fc As FileChooser                 			'Use filechooser from jFX library
	fc.Initialize                         			'Init! Do not forget
    fc.InitialDirectory = File.DirApp     			'Set starting folder
	fc.InitialFileName = "message.txt"				'Set an initial filename
    fc.Title = "Save File"
    Dim f As String = fc.ShowSave(frm)				'Show the dialog and select
	If f.Length = 0 Then Return						'If no file selected, return
	'Save the textarea to the file using textwriter
	Try
    	Dim tw As TextWriter
    	tw.Initialize(File.OpenOutput("", f, False))
    	tw.Write(taTextDialogMessage.Text)
    	tw.Flush
    	tw.Close
		lblTextDialogInformation.Text = "Message saved:"& f
	Catch
		lblTextDialogInformation.Text = "Error saving message:" & LastException.Message
      	If tw.IsInitialized Then tw.Close
  	End Try
End Sub

'Copy the TextDialog message content to the clipboard.
Sub btnTextDialogCopy_Action
	fx.Clipboard.SetString(taTextDialogMessage.Text)
End Sub
