﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=2.9
@EndOfDesignText@
'Class module
'File: fileutils.bas
'Purpose: file utility functions
'Info: This File Is part of the B4J Open Source Project todoreader.
'Read readme.txt for more information
'
Private Sub Class_Globals
	Private fx As JFX
	Public FindFilesList As List
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize
	FindFilesList.Initialize
End Sub

Public Sub SelectFolder(frm As Form, folder As String) As String              
  Dim dc As DirectoryChooser
  dc.Initialize
  dc.InitialDirectory = folder
  dc.Title = "Select a Folder"
  Dim folder As String
  folder = dc.Show(frm)
  Return folder 
End Sub

'Find files in a folder resursive
'Parameter Path to find files from
'Returns List of files in a global var FindFileList As List (must initialized)
'Example usage:
' Dim dir As String = File.DirApp
' FFileList.Clear
' FindFiles(dir)
' listviewFiles.Items.AddAll(FFileList)
Public Sub FindFiles(Path As String, sep As String)
  Dim FileList, FolderList As List
  Dim FileFound As String
  FileList.Initialize
  FolderList.Initialize
  If Path = "" Then Path = File.DirApp
	'Path = Path.Replace("\", "/")
  If File.IsDirectory(Path,"") Then
    Try
      FileList = File.ListFiles(Path)
      If FileList.IsInitialized Then
        If FileList.Size > 0 Then
          FileFound = Path
          FindFilesList.Add(FileFound)
          For i = 0 To FileList.Size - 1
            FileFound = FileList.Get(i)
            If File.IsDirectory(Path & sep & FileFound,"") Then
              FolderList.Add(Path & sep & FileFound)
            Else
              FindFilesList.Add(Path & sep & FileFound)
            End If
          Next
        End If
      End If
    Catch
      Log(LastException & " " & Path & " " & FileList.Size & " (File)")
    End Try
    'Recursive search for files
    Try
      If FolderList.IsInitialized Then
        If FolderList.Size > 0 Then
          For i = 0 To FolderList.Size - 1
            FileFound = FolderList.Get(i)
            FindFiles(FileFound, sep)
          Next
        End If
      End If
    Catch
      Log(LastException & " " & FolderList.Size & " " & FileFound & " (Folder)")
    End Try
  End If
End Sub

'Search for file with have .b4 in their extension
Public Sub FindB4Files(Path As String, sep As String)
	FindFilesList.Clear
	FindFiles(Path, sep)
	Dim l As List
	Dim s As String
	l.Initialize
	For i = 0 To FindFilesList.Size - 1
		s = FindFilesList.Get(i)
		If s.ToLowerCase.Contains(".meta") = False Then
			If s.ToLowerCase.Contains(".bak") = False Then
				If s.ToLowerCase.Contains(".b4") Then l.Add(s)
			End If
		End If
	Next
	FindFilesList.Clear
	FindFilesList.AddAll(l)
End Sub

'Search for file with have .xml in their extension
Public Sub FindXMLLibFiles(Path As String)
	Dim lall As List = File.ListFiles(Path)
	Dim lxml As List
	lxml.Initialize
	Dim s As String
	For i = 0 To lall.Size - 1
		s = lall.Get(i)
		'Option to add full path - NOT USED
		'If s.ToLowerCase.Contains(".xml") Then l.Add(s)
		'Option to add filename only instead of full path
		If s.ToLowerCase.Contains(".xml") Then lxml.Add(GetFileName(s))
	Next
	FindFilesList.Clear
	FindFilesList.AddAll(lxml)
End Sub

'Search for file with have .xml in their extension
Public Sub FindXMLLibFiles2(Path As String)
	Dim lall As List = File.ListFiles(Path)
	Dim lxml As List
	lxml.Initialize
	Dim s As String
	For i = 0 To lall.Size - 1
		s = lall.Get(i)
		'Option to add full path - NOT USED
		'If s.ToLowerCase.Contains(".xml") Then l.Add(s)
		'Option to add filename only instead of full path
		If s.ToLowerCase.Contains(".xml") Then 
			Dim f As String = GetFileName(s)
			Dim idx As Int = s.ToLowerCase.IndexOf(".xml")
			f = f.SubString2(0, idx)
			lxml.Add(f)
		End If
	Next
	FindFilesList.Clear
	FindFilesList.AddAll(lxml)
End Sub

'Returns the filename from a fullpath
Public Sub GetFileName(path As String) As String
  'Define the JavaObject
  Dim fileO As JavaObject
  'Get the fileio class
  fileO.InitializeNewInstance("java.io.File", Array As Object(path))
  'Getthe filename
  Return fileO.RunMethod("getName", Null)
End Sub

'Returns the filepath from a fullpath
Public Sub GetFilePath(path As String) As String
  'Define the JavaObject
  Dim fileO As JavaObject
  'Get the fileio class
  fileO.InitializeNewInstance("java.io.File", Array As Object(path))
  'Get the filename and path
  Dim n As String = fileO.RunMethod("getName", Null)
  Dim p As String = fileO.RunMethod("getPath", Null)
  p = p.Replace(n, "")
  Return p
End Sub

'Returns the last modified date & time from a fullpath
Public Sub GetFileLastModified(path As String) As String
	Dim d As Long
	d = File.LastModified(GetFilePath(path), GetFileName(path))
	Return DateTime.Date(d) & " " & DateTime.Time(d)
End Sub

'Replaces an item in a file. An item is a full line. The line is searched for. If found, then content will be replaced.
'Parameter: Folder
'Parameter: File
'Parameter: New Item
'Parameter: Linenumber
Public Sub ReplaceItemInFile(d As String, f As String, newitem As String, line As Int)
	If line < 1 Then Return
	line = line - 1
	Dim l As List
  	l.Initialize
	'Read the content of the file 
	Dim tr As TextReader        
  	Try
    	'init the textreader and read content into the list  
    	tr.Initialize(File.OpenInput(d, f))
    	l.AddAll(tr.ReadList)
		tr.Close                                                      
  	Catch
   		Log("Read Text File: Error " & LastException.Message)
    	If tr.IsInitialized Then tr.Close
  	End Try
	tr.Close
	'Change the line. If item is empty then remove line else replace content
	If newitem.Length = 0 Then
		l.RemoveAt(line)
	Else
		l.Set(line, newitem)
	End If
	'Write the revised list to the file again
	Dim tw As TextWriter
	Try
		tw.Initialize(File.OpenOutput(d, f, False))
		tw.WriteList(l)
		tw.Flush
    	tw.Close
  	Catch
    	Log("Write Text File: Error " & LastException.Message)
    	If tw.IsInitialized Then tw.Close
  	End Try
End Sub

'Get the content of a file and assign to a string
'Parameter: Full path to the file
'Returns the content of the file as string
Public Sub FileToString(path As String) As String
	Dim result As String = ""
	Dim tr As TextReader
	Try     
  		tr.Initialize(File.OpenInput(GetFilePath(path), GetFileName(path)) )
  		result = tr.ReadAll
	Catch
  		Log("Error loading form from file:" & LastException.Message)
	End Try
	Return result
End Sub

'Read the content of a textfile to a list
'Parameter: d = Folder, f = Textfile
'Returns: List. If error then List is empty
Sub FileToList(path As String) As List
	Dim tr As TextReader        
  	Dim l As List
  	l.Initialize
  	Try
    	'init the textreader and read content into the list  
  		tr.Initialize(File.OpenInput(GetFilePath(path), GetFileName(path)) )
    	l.AddAll(tr.ReadList)
    	tr.Close                                                      
  	Catch
    	Log("Read Text File: Error " & LastException.Message)
    	If tr.IsInitialized Then tr.Close
  	End Try
  	Return l
End Sub

'Get filename from full path
'Parameter FullPath: Full path to the file
'Paramater Del: The delimiter used, either / or \
Public Sub getFileName2(FullPath As String, Del As String) As String
	If Del = "" Then Del = "\"
	Return FullPath.SubString(FullPath.LastIndexOf(Del)+1)
End Sub

'Save the content of a textarea to a file
'Parameter: TextArea
Public Sub SaveTextArea(frm As Form, ta As TextArea)
	If ta.Text.Length = 0 Then Return		'If the textarea has no content, return
	Dim fc As FileChooser                 	'Use filechooser from jFX library
	fc.Initialize                         	'Init! Do not forget
    fc.InitialDirectory = File.DirApp     	'Set starting folder
	fc.InitialFileName = "test.txt"			'Set an initial filename
    fc.Title = "Save File"
    Dim f As String = fc.ShowSave(frm)		'Show the dialog and select
	If f.Length = 0 Then Return				'If no file selected, return
	'Save the textarea to the file using textwriter
	Try
    	Dim tw As TextWriter
    	tw.Initialize(File.OpenOutput("", f, False))
    	tw.Write(ta.Text)
    	tw.Flush
    	tw.Close
	Catch
		Log("Write Text File: Error " & LastException.Message)
      	If tw.IsInitialized Then tw.Close
  	End Try
End Sub
